/**        DBMS.java        */
package did;

import java.sql.*;
import java.util.*;
import java.io.*;
/**
 * Questa classe mette a disposizione i metodi per effettuare interrogazioni
 * sulla base di dati.
 */
public class DBMS {
	//Dati di identificazione dell'utente (da personalizzare)
    private String user = "userlabXX";
    private String passwd = "xxxxx";
	
    /** URL per la connessione alla base di dati � formato dai seguenti componenti:
     * <protocollo>://<host del server>/<nome base di dati>.
     */
    private String urldblab = "jdbc:postgresql://dbserver.sci.univr.it/dblabXX";

	/** Driver da utilizzare per la connessione e l'esecuzione delle query. */
    private String driver = "org.postgresql.Driver";

	//definizione delle Query 
	private String insertpic = "INSERT INTO peoplePicture (name, surname, picture) VALUES (?, ?, ?)";
	private String insertpath = "INSERT INTO peoplePicture (name, surname, picturepath) VALUES (?, ?, ?)";
	private String getpic =  "SELECT picture FROM peoplepicture WHERE id = ?";
	
	
    /**
     * Costruttore della classe. Carica i driver da utilizzare per la
     * connessione alla base di dati.
     *
     * @throws ClassNotFoundException Eccezione generata nel caso in cui
     *         i driver per la connessione non siano trovati nel CLASSPATH.
     */
    public DBMS() throws ClassNotFoundException {
		Class.forName(driver);
    }

	/** 
	* Metodo per l'inserimento nella base di dati di una persona e della sua foto
	*/
	public void storePeoplePicture(String name, String surname, File f) {
		Connection con = null;
		PreparedStatement pst = null;
		try {      
            con = DriverManager.getConnection(urldblab, user, passwd);
			pst = con.prepareStatement(insertpic);
			pst.clearParameters();
			pst.setString(1, name);
			pst.setString(2, surname);
			pst.setBinaryStream(3, new FileInputStream(f), (int)f.length());
			pst.execute(); 
			con.close();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	/** 
	* Metodo per l'inserimento nella base di dati di una persona e del path della sua foto
	*/
	public void storePeoplePicture(String name, String surname, String path) {
		Connection con = null;
		PreparedStatement pst = null;
		try {
            con = DriverManager.getConnection(urldblab, user, passwd);
			pst = con.prepareStatement(insertpath);
			pst.clearParameters();
			pst.setString(1, name);
			pst.setString(2, surname);
			pst.setString(3, path);
			pst.execute(); 
			con.close();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}

	/** 
	* Metodo per la ricerca della foto della persona identificata da id.
	* @return l'InputStream da cui e' possibile leggere la foto memorizzata nel DB.
	*/
	public InputStream searchPicture(int id) {
		PreparedStatement pstmt = null;
		Connection con = null;
		ResultSet rs = null;
		InputStream is = null;	
		try {
			con = DriverManager.getConnection(urldblab, user, passwd);
			pstmt = con.prepareStatement(getpic); 
			pstmt.clearParameters();
			pstmt.setInt(1, id);
			rs = pstmt.executeQuery();
			rs.next();
			//l'immagine, di tipo bytea nel DB, viene ottenuta come un binary stream, in particolare un InputStream
			is = rs.getBinaryStream("picture");
		} catch(SQLException sqle) {                /* Catturo le eventuali eccezioni! */
			sqle.printStackTrace();
		} finally {                                 /* Alla fine chiudo la connessione. */
			try {
				con.close();
			} catch(SQLException sqle1) {
				sqle1.printStackTrace();
			}
		}
		return is;
	}

	/** 
	* Metodo per la ricerca di tutte le persone (e delle loro foto) con nome e cognome specificati.
	* @return il Vector contenente tutte le info, tranne le eventuali immagini memorizzate direttamente nel DB, delle persone recuperate.
	*/
	public Vector search(String name, String surname) {
		Vector result = new Vector();
		ResultSet rs = null;
		PreparedStatement pstmt = null;
		Connection con = null;
		//preparo la stringa della query da eseguire
		String search = "SELECT id,name,surname,picturepath FROM peoplepicture";
		//la query e' costruita dipendentemente dalla presenza o meno dei parametri "name" e "surname"
		if (!name.isEmpty() || !surname.isEmpty()) search += " WHERE";
		if (!name.isEmpty()) search += " name = ?";
		if (!name.isEmpty() && !surname.isEmpty()) search += " AND";
		if (!surname.isEmpty()) search += " surname = ?";
		try {
			con = DriverManager.getConnection(urldblab, user, passwd);
			pstmt = con.prepareStatement(search); 
			pstmt.clearParameters();
			int i = 1;
			//imposto i parametri nella query
			//i parametri dipendono dalla presenza o meno dei parametri "name" e "surname"
			if (!name.isEmpty()) {
				pstmt.setString(i, name); 
				i++;
			}
			if (!surname.isEmpty()) pstmt.setString(i, surname); 
			rs = pstmt.executeQuery();
			// Memorizzo il risultato dell'interrogazione nel Vector
			while(rs.next())
				result.add(makePeoplePictureBean(rs));
		} catch(SQLException sqle) {                /* Catturo le eventuali eccezioni! */
			sqle.printStackTrace();
		} finally {                                 /* Alla fine chiudo la connessione. */
			try {
				con.close();
			} catch(SQLException sqle1) {
				sqle1.printStackTrace();
			}
		}
		return result;
	}

	//Metodo per la creazione di un bean a partire dal record attuale del ResultSet dato come parametro
	private PeoplePictureBean makePeoplePictureBean(ResultSet rs) throws SQLException {
		PeoplePictureBean bean = new PeoplePictureBean();
		bean.setId(rs.getInt("id"));
		bean.setName(rs.getString("name"));
		bean.setSurname(rs.getString("surname"));
		bean.setPicturePath(rs.getString("picturepath"));
		return bean;
    }
	
}
