import java.io.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import com.oreilly.servlet.MultipartRequest;
import did.*;

/**
 * Questa classe gestisce tutte le richieste, ottiene eventuali dati dal DB tramite la classe DBMS 
 * e poi visualizza eventuali risultati passando il controllo ad un'apposita JSP.
 * Risponde a richieste HTTP di tipo GET e POST. 
 */
public class photos extends HttpServlet {
    /**
     * Questo metodo risponde alle richieste HTTP di tipo GET. Elabora le richieste, impostando
     * gli eventuali attributi necessari, e ridirige la visualizzazione alla pagina JSP.
     *
     * @param request Oggetto HttpServletRequest dal quale ottenere informazioni circa la
     *                richiesta effettuata.
     * @param response Oggetto HttpServletResponse per l'invio delle risposte.
	 * I possibili parametri che vengono considerati e le relative azioni effettuate sono le
	 * seguenti:
	 * <br>
	 * <ol>
	 *   <li>nessun parametro: viene visualizzata la pagina di ricerca e upload;</li>
	 *   <li>parametro 'id': viene restituita la foto associata alla persona identificata da id;</li>
	 *   <li>parametro 'path': viene restituita la foto memorizzata al percorso specificato.</li>
	 * </ol>
     */
	public void doGet(HttpServletRequest request, HttpServletResponse response) 
	throws IOException, ServletException {
		String id = request.getParameter("id");
		String path = request.getParameter("path");
		RequestDispatcher dispatcher = null;

		//quando non e' presente nessun parametro visualizzo le form di ricerca e upload
		if (id==null && path==null) {
			//richiamo la jsp di upload/search
			dispatcher = getServletContext().getRequestDispatcher("/forms.html");
        	dispatcher.forward(request, response);
		}

		//se e' presente l'id recupero e restituisco l'immagine associata alla persona identificata da id
		if (id!=null) {
			//ottengo lo stream di output verso la JSP
			PrintWriter out = response.getWriter(); 
			try {
				int i;
				DBMS dbms = new DBMS();
				//la classe DBMS restituisce un InputStream con cui costruisco un buffered input stream
				BufferedInputStream bis = new BufferedInputStream(dbms.searchPicture(Integer.parseInt(id)));
				//imposto il tipo della risposta alla JSP
				response.setContentType("image/jpeg");
				//imposto la dimensione in byte della risposta alla JSP
				response.setContentLength(bis.available());
				//byte per byte copio l'immagine letta dal DB sullo stream verso la JSP
				while ((i = bis.read()) != -1) 
					out.write(i);
				//chiudo lo stream in lettura
				bis.close();
			} catch (Exception ex) {
				//se viene lanciata qualche eccezione ne visualizzo il messaggio di errore tramite la JSP
				request.setAttribute("msg", ex.toString());
				dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
				dispatcher.forward(request, response);
				//e visualizzo lo stack trace completo sullo standard error (di solito logs/catalina.out)
				ex.printStackTrace();
			}
		}
		
		//se e' presente il path di un'immagine la leggo e la restituisco alla JSP
		if (path!=null) {
			//ottengo lo stream di output verso la JSP
			PrintWriter out = response.getWriter(); 
			try {
				int i;
				//leggo l'immagine dal disco tramite un file input stream
				FileInputStream fis = new FileInputStream(path);
				//imposto il tipo della risposta alla JSP
				response.setContentType("image/jpeg");
				//imposto la dimensione in byte della risposta alla JSP
				response.setContentLength(fis.available());
				//byte per byte copio l'immagine letta dal DB sullo stream verso la JSP
				while ((i = fis.read()) != -1) 
					out.write(i);
				//chiudo lo stream in lettura
				fis.close();
			} catch (Exception ex) {
				//se viene lanciata qualche eccezione ne visualizzo il messaggio di errore tramite la JSP
				request.setAttribute("msg", ex.toString());
				dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
				dispatcher.forward(request, response);
				//e visualizzo lo stack trace completo sullo standard error (di solito logs/catalina.out)
				ex.printStackTrace();
			}
		}
	}

    /**
     * Questo metodo risponde alle richieste HTTP di tipo POST. Elabora le richieste, impostando
     * gli eventuali attributi necessari, e ridirige la visualizzazione alla pagina JSP.
     *
     * @param request Oggetto HttpServletRequest dal quale ottenere informazioni circa la
     *                richiesta effettuata.
     * @param response Oggetto HttpServletResponse per l'invio delle risposte.
	 * L'azione eseguita e' controllata dal parametro "command". I suoi possibili valori che vengono considerati e le relative azioni effettuate sono:
	 * <br>
	 * <ol>
	 *   <li>'SEARCH': vengono cercate le info su tutte le persone con nome e cognome corrispondenti agli ulteriori
	 *                 parametri "name" e "surname";</li>
	 *   <li>'UPLOAD': vengono memorizzate nel DB le info inserite nella FORM. 
	 *                 Se il checkbox "storeDB" e' impostato a VERO allora l'immagine viene inserita direttamente nel DB.
	 *                 Altrimenti l'immagine viene salvata nell'apposita cartella "uploads" e nel DB ne viene inserito il path.</li>
	 * </ol>
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
	throws IOException, ServletException {

		//Definizione e recupero dell'eventuale parametro della servlet
		//Dichiaro l'oggetto Dispatcher necessario per passare il controllo ad una JSP o una pagina HTML
		RequestDispatcher dispatcher = null;
		//Essendo i parametri della form di tipo Multipart, ottengo l'oggetto MultipartRequest da cui e' possibile ottenere eventuali parametri
		MultipartRequest multi = new MultipartRequest(request,"/tmp/");
		//ottengo il parametro command che controlla quale azione deve essere eseguita
		String command = (String)multi.getParameter("command");

		try {
			// Oggetto per l'interazione con il Database
			DBMS dbms = new DBMS();
			// se il comando e' SEARCH
			if (command.equals("SEARCH")) {
				//ottengo gli eventuali ulteriori parametri NAME e SURNAME
				String name = multi.getParameter("NAME");
				String surname = multi.getParameter("SURNAME");
				//tramite la classe DBMS ricerco nel DB le informazioni richieste
				Vector result = dbms.search(name,surname);
				if (result.size()==0) { //se non sono state trovate informazioni che rispettino i vincoli dati visualizzo l'apposito messaggio
										//passando il controllo alla JSP
					request.setAttribute("msg", "Sorry, no data has been found.");
					dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
					dispatcher.forward(request, response);
				}
				else {		//se sono state trovate informazioni le inoltro alla JSP perche' le visualizzi
					request.setAttribute("data", result);
					dispatcher = getServletContext().getRequestDispatcher("/view.jsp");
					dispatcher.forward(request, response);
				}
			}
						
			// se il comando e' UPLOAD
			if (command.equals("UPLOAD")) { 
				//ottengo gli eventuali ulteriori parametri NAME, SURNAME, storeDB e IMAGE
				String name = multi.getParameter("NAME");
				String surname = multi.getParameter("SURNAME");
				String[] store = multi.getParameterValues("storeDB");
				File f = multi.getFile("IMAGE");

				// se name o surname non sono stati inseriti, tramite la JSP, visualizzo un messaggio di errore
				if (name.isEmpty() || surname.isEmpty()) {
					request.setAttribute("msg", "Please, specify BOTH name AND surname.");
					dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
					dispatcher.forward(request, response);
				} else // se non e' stato specificata l'immagine da memorizzare, tramite la JSP, visualizzo un messaggio di errore
					if (f==null) {
						request.setAttribute("msg", "Please, specify a file name.");
						dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
						dispatcher.forward(request, response);
					} else { //se tutti i parametri sono stati specificati nella FORM...
						//ottengo il tipo del file/immagine inserita nella FORM
							String fileName = multi.getFilesystemName("IMAGE");
							if (store==null) {//memorizzo nel DB una nuova tupla con il nome e il cognome specificati e il path all'immagine
								//costruisco il path assoluto in cui memorizzare l'immagine
								//il metodo System.getenv() permette di recuperare il valore di una variabile d'ambiente
								//in questo caso ottengo il valore della variabile CATALINA_BASE che noi abbiamo impostato in ".bashrc"
								//il file viene memorizzato in un'apposita cartella "uploads"
								String filepath = System.getenv("CATALINA_BASE") + "/uploads/" + fileName;
								File fOUT = new File(filepath);
								//Apro i file stream in ingresso (da cui leggere il file originale)...
								FileInputStream fIS = new FileInputStream(f);
								//...e in uscita (su cui scrivere l'immagine)
								FileOutputStream fOS = new FileOutputStream(fOUT);
								//copio byte per byte l'immagine dallo stream in ingresso a quello in uscita
								while (fIS.available()>0)
									fOS.write(fIS.read());
								//chiudo gli stream
								fIS.close();
								fOS.close();
								//inserisco nel db la nuova tupla
								dbms.storePeoplePicture(name,surname,filepath);
								//richiamo la JSP per visualizzare la conferma dell'upload/inserimento
								request.setAttribute("msg", "File "+fileName+" has been saved in "+filepath+"<BR>and associated to "+name+" "+surname);
								dispatcher = getServletContext().getRequestDispatcher("/confirm.jsp");
								dispatcher.forward(request, response);
							} else { //memorizzo nel DB una nuova tupla con il nome e il cognome specificati e inserisco nel DB direttamente l'immagine scelta
								dbms.storePeoplePicture(name,surname,f);
								//richiamo la JSP per visualizzare la conferma dell'upload/inserimento
								request.setAttribute("msg", "File "+fileName+" has been uploaded in the DB and associated to "+name+" "+surname);
								dispatcher = getServletContext().getRequestDispatcher("/confirm.jsp");
								dispatcher.forward(request, response);
							}
					}
			}			
		} catch(Exception ex) {
			//se viene lanciata qualche eccezione ne visualizzo il messaggio di errore tramite la JSP
			request.setAttribute("msg", ex.toString());
			dispatcher = getServletContext().getRequestDispatcher("/error.jsp");
			dispatcher.forward(request, response);
			ex.printStackTrace();
		}
    }
}
